
######################################
######    Basic information    #######
######################################

# R code for the article "Number agreement, dependency length, and word order in Finnish traditional dialects"
# Published in: Proceedings of the Sixth International Conference on Dependency Linguistics (Depling, SyntaxFest 2021). Stroudsburg, PA: The Association for Computational Linguistics.
# Author, article: Kaius Sinnemäki & Akira Takaki
# Author, R code: Kaius Sinnemäki
# Most recent update: 8 December 2021

### Load necessary packages to R session
# pkgs = c("tidyverse", "ggplot2", "cowplot", "blme", "MuMIn", "sjPlot")
# install.packages("pkgs")

library(tidyverse)
library(ggplot2)
library(cowplot)
library(blme)
library(MuMIn)
library(sjPlot)


### Read dialect data into the session and preprocess it
# Read data and make a copy to serve as the workhorse
master <- read.csv("data.csv")
df = master

# Preprocess the variables
df$agreement.Vfin = factor(df$agreement.Vfin, levels = c("sg", "pl"))
df$agreement.Vlex = factor(df$agreement.Vlex, levels = c("sg", "pl"))

df$dlength_fin = df$distance.Vfin
df$dlength_fin[df$order.Vfin=="sv"] = 0-(df$distance.Vfin[df$order.Vfin=="sv"]+1)
df$dlength_fin[df$order.Vfin=="vs"] = df$distance.Vfin[df$order.Vfin=="vs"]+1

df$dlength_lex = df$distance.Vlex
df$dlength_lex[df$order.Vlex=="sv"] = 0-(df$distance.Vlex[df$order.Vlex=="sv"]+1)
df$dlength_lex[df$order.Vlex=="vs"] = df$distance.Vlex[df$order.Vlex=="vs"]+1

# Descriptive statistics
df %>%
  group_by(agreement.Vfin) %>%
  summarize(n=n()) %>%
  mutate(freq = n/sum(n))
df %>%
  group_by(order.Vfin) %>%
  summarize(n=n()) %>%
  mutate(freq = n/sum(n))
range(df$dlength_fin)

df %>%
  group_by(agreement.Vlex) %>%
  summarize(n=n()) %>%
  drop_na() %>% # in present tense negative polarity, the main lexical verb does not agree with the subject in number; these instances are removed
  mutate(freq = n/sum(n))
df %>%
  group_by(order.Vlex) %>%
  summarize(n=n()) %>%
  drop_na() %>% # in present tense negative polarity, the main lexical verb does not agree with the subject in number; these instances are removed
  mutate(freq = n/sum(n))
range(df$dlength_lex)



### Modelling

## Random intercept models
# Models with the finite verb as the reference point
m.fin.0 <- bglmer(agreement.Vfin ~ 1 + 
                    (1|lemma.noun) +
                    (1|dialect.group/interviewee) +
                    (1|lemma.Vlex),
                  data=df, family="binomial",
                  control = glmerControl(optimizer = "bobyqa"))
m.fin.1 <- bglmer(agreement.Vfin ~ 1 + dlength_fin + 
               (1|lemma.noun) +
               (1|dialect.group/interviewee) +
               (1|lemma.Vlex),
             data=df, family="binomial",
             control = glmerControl(optimizer = "bobyqa"))

summary(m.fin.1)
anova(m.fin.0, m.fin.1)
AIC(m.fin.0)-AIC(m.fin.1)
(r2.fin <- r.squaredGLMM(m.fin.1))

# Models with the lexical verb as the reference point
m.lex.0 <- bglmer(agreement.Vlex ~ 1 + 
                (1|lemma.noun) + 
                (1|dialect.group/interviewee) +
                (1|lemma.Vlex),
              data=df, family="binomial",
              control = glmerControl(optimizer = "bobyqa"))
m.lex.1 <- bglmer(agreement.Vlex ~ 1 + dlength_lex + 
               (1|lemma.noun) + 
               (1|dialect.group/interviewee) +
               (1|lemma.Vlex),
             data=df, family="binomial",
             control = glmerControl(optimizer = "bobyqa"))
summary(m.lex.1)
anova(m.lex.0, m.lex.1)
AIC(m.lex.0)-AIC(m.lex.1)
(r2.lex <- r.squaredGLMM(m.lex.1))


## Random slope models

# Select only those dialect areas where there is enough
# data for modelling. The criteria are that the frequency of
# the lower value of the response variable (i.e., plural) >= 20
# and that its proportion >= 0.1.
df %>%
  group_by(dialect.group, agreement.Vfin) %>%
  summarize(n=n()) %>%
  mutate(freq = n/sum(n)) %>%
  filter(n >= 20 & freq >= 0.1) %>%
  filter(agreement.Vfin == "pl") %>%
  select(dialect.group) -> keep.Vfin
df.slope.Vfin = df[(df$dialect.group %in% keep.Vfin$dialect.group),]
df.slope.Vfin$dialect.group = factor(df.slope.Vfin$dialect.group)
levels(df.slope.Vfin$dialect.group) = c("South Karelia", "Central Karelia", "North Karelia", "Länsipohja", "Peräpohjola", "Southwestern: south", "Southwestern: north", "Turku uplands")
df.slope.Vfin$dialect.group = as.character(df.slope.Vfin$dialect.group)

df %>%
  group_by(dialect.group, agreement.Vlex) %>%
  summarize(n=n()) %>%
  drop_na() %>% # in present tense negative polarity, the main lexical verb does not agree with the subject in number; these instances are removed
  mutate(freq = n/sum(n)) %>%
  filter(n >= 20 & freq >= 0.1) %>%
  filter(agreement.Vlex == "pl") %>%
  select(dialect.group) -> keep.Vlex
df.slope.Vlex = df[(df$dialect.group %in% keep.Vlex$dialect.group),]
df.slope.Vlex$dialect.group = factor(df.slope.Vlex$dialect.group)
levels(df.slope.Vlex$dialect.group) = c("South Karelia", "Central Karelia", "North Karelia", "Länsipohja", "Peräpohjola", "Central Ostrobothnia", "North Ostrobothnia", "Southwestern: south", "Southwestern: north", "Turku uplands")
df.slope.Vlex$dialect.group = as.character(df.slope.Vlex$dialect.group)

# Models with the finite verb as the reference point
m.fin.0.sl <- bglmer(agreement.Vfin ~ 1 + 
                    (1|lemma.noun) +
                    (1+dlength_fin|dialect.group/interviewee) +
                    (1|lemma.Vlex),
                  data=df.slope.Vfin, family="binomial",
                  control = glmerControl(optimizer = "bobyqa"))
m.fin.1.sl <- bglmer(agreement.Vfin ~ 1 + dlength_fin + 
                    (1|lemma.noun) +
                    (1+dlength_fin|dialect.group/interviewee) +
                    (1|lemma.Vlex),
                  data=df.slope.Vfin, family="binomial",
                  control = glmerControl(optimizer = "bobyqa"))

summary(m.fin.1.sl)
anova(m.fin.0.sl, m.fin.1.sl)
AIC(m.fin.0.sl)-AIC(m.fin.1.sl)
(r2.fin.sl <- r.squaredGLMM(m.fin.1.sl))

# Models with the lexical verb as the reference point
m.lex.0.sl <- bglmer(agreement.Vlex ~ 1 + 
                    (1|lemma.noun) + 
                    (1+dlength_lex|dialect.group/interviewee) +
                    (1|lemma.Vlex),
                  data=df.slope.Vlex, family="binomial",
                  control = glmerControl(optimizer = "bobyqa"))
m.lex.1.sl <- bglmer(agreement.Vlex ~ 1 + dlength_lex + 
                    (1|lemma.noun) + 
                    (1+dlength_lex|dialect.group/interviewee) +
                    (1|lemma.Vlex),
                  data=df.slope.Vlex, family="binomial",
                  control = glmerControl(optimizer = "bobyqa"))
summary(m.lex.1.sl)
anova(m.lex.0.sl, m.lex.1.sl)
AIC(m.lex.0.sl)-AIC(m.lex.1.sl)
(r2.lex.sl <- r.squaredGLMM(m.lex.1.sl))



### Figures

# Fig 1
# Inspiration for the plot arrangement was taken from member s_ at
# https://stackoverflow.com/questions/61517820/placement-of-legend-using-plot-grid-of-cowplot

df %>%
  ggplot(aes(x=dlength_fin, color=agreement.Vfin, fill=agreement.Vfin)) +
  geom_histogram(aes(y=..density..), position="identity", 
                 alpha=0.5, bins = 20) + 
  geom_density(alpha=0.3) +
  scale_color_manual(guide = "none", values=c("#999999", "#E69F00"))+
  scale_fill_manual(name = "Agreement",
                    labels = c("singular", "plural"),
                    values = c("#999999", "#E69F00"))+
  labs(title="Finite verb", x="Dependency length", 
       y = "Density") + 
  theme_bw() -> p.hist.fin

df %>%
  drop_na() %>% # in present tense negative polarity, the main lexical verb does not agree with the subject in number; these instances are removed
  ggplot(aes(x=dlength_lex, color=agreement.Vlex, fill=agreement.Vlex)) +
  geom_histogram(aes(y=..density..), position="identity", 
                 alpha=0.5, bins = 20) +
  geom_density(alpha=0.3) +
  scale_color_manual(guide = "none", values=c("#999999", "#E69F00"))+
  scale_fill_manual(name = "Agreement",
                    labels = c("singular", "plural"),
                    values = c("#999999", "#E69F00"))+
  labs(title="Lexical verb", x="Dependency length", 
       y = "Density") + 
  theme_bw() -> p.hist.lex


legend <- cowplot::get_legend(p.hist.fin + theme(legend.position = 'bottom'))
bottom_row <- cowplot::plot_grid(legend)
upper_row <- cowplot::plot_grid(
  p.hist.fin + theme(legend.position = 'none'),
  p.hist.lex + theme(legend.position = 'none'),
  ncol =2, labels = c('A', 'B'))

jpeg(file="fig1.jpg",res=150,width=1600, height=500)
cowplot::plot_grid(upper_row,
                   bottom_row, 
                   labels = c('',''), label_size = 12, ncol = 1, rel_heights = c(6,1))
dev.off()

# Fig 2
m.fin.1 %>%
  plot_model(type = "pred", terms = "dlength_fin[all]",
                    axis.title = c("Dependency length",
                                   "Predicted probability\nof plural agreement"),
                    title = "Finite verb", show.data = T,
                    line.size = 1.5, jitter = 0.02) + 
  scale_y_continuous(breaks = c(0,0.2,0.4,0.6,0.8,1.0),
                     limits = c(-0.02,1.02),
                     labels = scales::percent) +
  theme(axis.text.x = element_text(size = 12),
        axis.text.y = element_text(size = 12),
        axis.title.x = element_text(size = 14),
        axis.title.y = element_text(size = 14),
        plot.title = element_text(size=16))  + 
  theme_bw() -> p.fin

m.lex.1 %>%
  plot_model(type = "pred", terms = "dlength_lex[all]",
                    axis.title = c("Dependency length",
                                   "Predicted probability\nof plural agreement"),
                    title = "Lexical verb", show.data = T,
                    line.size = 1.5, jitter = 0.02) + 
  scale_y_continuous(breaks = c(0,0.2,0.4,0.6,0.8,1.0),
                     limits = c(-0.02,1.02),
                     labels = scales::percent) +
  theme(axis.text.x = element_text(size = 12),
        axis.text.y = element_text(size = 12),
        axis.title.x = element_text(size = 14),
        axis.title.y = element_text(size = 14),
        plot.title = element_text(size=16))  + 
  theme_bw() -> p.lex


jpeg(file="fig2.jpg",res=150,width=1600, height=500)
cowplot::plot_grid(p.fin, p.lex, ncol=2, nrow=1,
                   labels = c("A", "B"))

dev.off()

# Fig 3
m.fin.1.sl %>%
  plot_model(type = "pred", terms = c("dlength_fin[all]","dialect.group"),
                       pred.type = "re",
                       axis.title = c("Dependency length",
                                   "Predicted probability of plural agreement"),
                    title = "Finite verb",
                    line.size = 1, ci.lvl = F, grid=T,
                    show.data = T, colors = "bw",
                    jitter = 0.02) + 
  scale_y_continuous(breaks = c(0,0.25,0.5,0.75,1.0),
                     limits = c(-0.02,1.02),
                     labels = scales::percent) +
  theme(axis.text.x = element_text(size = 12),
        axis.text.y = element_text(size = 12),
        axis.title.x = element_text(size = 14),
        axis.title.y = element_text(size = 14),
        plot.title = element_text(size=16),
        strip.text.x = element_text(size = 9.5))  + 
  theme_bw() -> p.fin.sl

m.lex.1.sl %>%
  plot_model(type = "pred", terms = c("dlength_lex[all]","dialect.group"),
                       pred.type = "re",
                       axis.title = c("Dependency length",
                                   "Predicted probability of plural agreement"),
                    title = "Lexical verb",
                    line.size = 1, ci.lvl = F, grid=T,
                    show.data = T, colors = "bw",
                    jitter = 0.02) + 
  scale_y_continuous(breaks = c(0,0.25,0.5,0.75,1.0),
                     limits = c(-0.02,1.02),
                     labels = scales::percent) +
  theme(axis.text.x = element_text(size = 12),
        axis.text.y = element_text(size = 12),
        axis.title.x = element_text(size = 14),
        axis.title.y = element_text(size = 14),
        plot.title = element_text(size=16),
        strip.text.x = element_text(size = 9.5))  + 
  theme_bw() -> p.lex.sl


jpeg(file="fig3.jpg",res=150,width=1600, height=800)
cowplot::plot_grid(p.fin.sl, p.lex.sl, ncol=2, nrow=1,
                   rel_widths = c(1,1.3), labels = c("A", "B"))
dev.off()

